/*
    Copyright (C) 1999 Paul Barton-Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: configuration.cc,v 1.26 2000/11/03 04:25:18 pbd Exp $
*/

#include "configuration.h"

Configuration *Config;

Configuration::Configuration ()
	: guileconfig ("ardour")

{
	set_defaults ();
	
	variables["audio-device"] = &audio_device;
	variables["midi-device"] = &midi_device;
	variables["use-real-time"] = &use_real_time;
	variables["real-time-priority"] = &real_time_priority;
	variables["pixmap-path"] = &pixmap_path;
	variables["rec-enable-pix-regexp"] = &rec_enable_pix_regexp;
	variables["monitor-pix-regexp"] = &monitor_pix_regexp;
	variables["toggle-button-pix-regexp"] = &toggle_button_pix_regexp;
	variables["push-button-pix-regexp"] = &push_button_pix_regexp;
	variables["small-push-button-pix-regexp"] = &small_push_button_pix_regexp;
	variables["frames-per-cycle"] = &frames_per_cycle;
	variables["sample-rate"] = &sample_rate;
	variables["tape-path"] = &tape_path;
	variables["max-tracks"] = &max_tracks;
	variables["smpte-frames-per-second"] = &smpte_frames_per_second;
	variables["over-length-short"] = &over_length_short;
	variables["over-length-long"] = &over_length_long;
	variables["minimum-disk-io-bytes"] = &minimum_disk_io_bytes;
	variables["track-buffer-seconds"] = &track_buffer_seconds;
	variables["default-tape"] = &default_tape;
	variables["systemic-latency"] = &systemic_latency;
	variables["send-midi-timecode"] = &send_midi_timecode;
	variables["shuttle-speed-factor"] = &shuttle_speed_factor;
	variables["shuttle-speed-threshold"] = &shuttle_speed_threshold;
	variables["rf-samples"] = &rf_samples;
	variables["ui-rcfile"] = &ui_rcfile;
	variables["meter-hold"] = &meter_hold;
	variables["tape-button-image"] = &tape_button_image;
	variables["play-button-image"] = &play_button_image;
	variables["forward-button-image"] = &forward_button_image;
	variables["reverse-button-image"] = &reverse_button_image;
	variables["record-button-image"] = &record_button_image;
	variables["stop-button-image"] = &stop_button_image;
	variables["start-button-image"] = &start_button_image;
	variables["end-button-image"] = &end_button_image;
	variables["set-next-button-image"] = &set_next_button_image;
	variables["max-level"] = &max_level;
	variables["min-level"] = &min_level;
	variables["click-emphasis-sound"] = &click_emphasis_sound;
	variables["click-sound"] = &click_sound;
}

Configuration::~Configuration ()

{
}
	
int
Configuration::load_rcfile ()

{
	return bootstrap ();
}

void
Configuration::set_defaults ()

{
	use_real_time = false;
	real_time_priority = 10;
	pixmap_path = ".";
	rec_enable_pix_regexp = "onoff-button[0-9]*.xpm$";
	monitor_pix_regexp = "monitor-button[0-9]*.xpm$";
	toggle_button_pix_regexp = "led-button-[0-9]*.xpm$";
	push_button_pix_regexp = "no-led-button-[0-9]*.xpm$";
	small_push_button_pix_regexp = "small-no-led-button-[0-9]*.xpm$";
	tape_button_image = "tape-button.xpm$";
	play_button_image = "play_button.xpm";
	forward_button_image = "forward_button.xpm";
	reverse_button_image = "reverse_button.xpm";
	record_button_image = "record_button.xpm";
	stop_button_image = "stop_button.xpm";
	start_button_image = "start_button.xpm";
	end_button_image = "end_button.xpm";
	end_button_image = "set_next_button.xpm";
	frames_per_cycle = 1024;
	sample_rate = 48000;
	systemic_latency = 0;
	tape_path = "";
	default_tape = "";
	max_tracks = 24;
	smpte_frames_per_second = 30.0f;
	over_length_short = 2;
	over_length_long = 10;
	minimum_disk_io_bytes = 1024 * 64;
	track_buffer_seconds = 2.0;
	send_midi_timecode = true;
	shuttle_speed_factor = 1.0;
	shuttle_speed_threshold = -1; /* never scale shuttle speed */
	rf_samples = 1024; /* whatever */
	ui_rcfile = "./ardour_ui.rc";
	meter_hold = 99999;
	max_level = 0; /* see example file for explanation */
	min_level = 0; /* see example file for explanation */
	click_sound = "sounds/click.raw";
	click_emphasis_sound = "sounds/click_emphasis.raw";
}

int
Configuration::post_parse ()

{
	return 0;
}
