/*
    Copyright (C) 2000 Paul Barton-Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: riff.cc,v 1.3 2000/06/10 17:39:32 pbd Exp $
*/

#include <unistd.h>

#include "riff.h"

const int riff_header_size = 44;

int
write_riff_header (int fd,
		   short wavefmt, 
		   short channels,
		   int srate,
		   short bytes_per_sample,
		   short bits_per_sample,
		   size_t file_length)
		   
{
	int x;
	short s;

	/*

           Offset    Description
           ------    -----------
            0x00  00     chunk id 'RIFF'
            0x04  04     chunk size (32-bits)
            0x08  08     wave chunk id 'WAVE'
            0x0C  12     format chunk id 'fmt '
            0x10  16     format chunk size (32-bits)
            0x14  20     format tag (currently pcm)
            0x16  22     number of channels 1=mono, 2=stereo
            0x18  24     sample rate in hz
            0x1C  28     average bytes per second
            0x20  32     number of bytes per sample
                          1 =  8-bit mono
                          2 =  8-bit stereo or
                              16-bit mono
                          4 = 16-bit stereo
            0x22  34     number of bits in a sample
            0x24  36     data chunk id 'data'
            0x28  40     length of data chunk (32-bits)
            0x2C  44     Sample data

	*/

	lseek (fd, 0, SEEK_SET);
	write (fd, "RIFF", 4);  /* + 0 */
	write (fd, &file_length, 4); /* + 4 */
	write (fd, "WAVE", 4);  /* + 8 */
	write (fd, "fmt ", 4);  /* + 12 */
	x = 16;                 /* length of `fmt' chunk */
	write (fd, &x, 4); /* + 16 */
	write (fd, &wavefmt, 2); /* + 20 */
	write (fd, &channels, 2); /* + 22 */
	write (fd, &srate, 4); /* + 24 */
	x = srate * bytes_per_sample;
 	write (fd, &x, 4);  /* + 28 rate */
	write (fd, &bytes_per_sample, 2); /* + 32 */
	write (fd, &bits_per_sample, 2);  /* + 34 */
	s = 0;

	/* start the data chunk */

	write (fd, "data", 4); /* + 36 */
	x = file_length - riff_header_size;
	write (fd, &x, 4); /* + 40 */

	return 0;
}

int
read_riff_header (int fd,
		  short *wavefmt, 
		  short *channels,
		  int *srate,
		  short *bytes_per_sample,
		  short *bits_per_sample,
		  size_t *file_length)
		   
{
	lseek (fd, 4, SEEK_SET);
	read (fd, file_length, 4);
	lseek (fd, 20, SEEK_SET);
	read (fd, wavefmt, 2);
	read (fd, channels, 2);
	read (fd, srate, 4);
	lseek (fd, 4, SEEK_CUR); /* skip bytes per second */
	read (fd, bytes_per_sample, 2);
	read (fd, bits_per_sample, 2);

	return 0;
}



